/**
* \file RegistryService.h
*
* \brief API do servio de registro.
*
*/

#ifndef REGISTRYSERVICE_H_
#define REGISTRYSERVICE_H_

#include "../stubs/scs.hh"
#include "../stubs/registry_service.hh"

/**
* \brief Stubs dos servios bsicos.
*/
namespace openbusidl {

/**
* \brief Stub do servio de registro.
*/
  namespace rs {

  /**
  * \typedef Facet
  * \brief Representa uma facet atravs de uma string.
  */

  /**
  * \class FacetList
  * \brief Lista de facetas.
  */

  /**
  * \class PropertyValue
  * \brief Representa um valor de uma propriedade. 
  *
  * Este valor  uma sequncia de strings, no caso, o valor pode ser nico, 
  * sendo representado portanto por somente uma string.
  *
  */

  /**
  * \struct Property
  * \brief Representa uma propriedade que  um par chave/valor.
  *
  * A chave deve ser uma string. O valor pode ser nico ou mltiplo, sendo
  * representado atravs de uma sequncia de strings. \see PropertyValue
  *
  */

  /**
  * \class PropertyList
  * \brief Lista de propriedades.
  */

  /**
  * \class ServiceOffer
  * \brief Oferta de servio
  *
  * Descreve um servio atravs de uma lista de propriedades e um componente 
  * SCS. \see PropertyList
  */

  /**
  * \class ServiceOfferList
  * \brief Lista de ofertas de servio.
  */

  }
}

/**
* \brief openbus
*/
namespace openbus {
/**
* \brief Servios bsicos do Openbus.
*/
  namespace services {

    typedef openbusidl::rs::FacetList FacetList;
    typedef openbusidl::rs::PropertyList PropertyList;
    typedef openbusidl::rs::ServiceOffer ServiceOffer;
    typedef openbusidl::rs::ServiceOfferList ServiceOfferList;
    typedef openbusidl::rs::ServiceOfferList_var ServiceOfferList_var;

  /**
  * \brief Auxilia na construo de uma lista de facetas.
  */
    class FacetListHelper {
      private:
        openbusidl::rs::FacetList_var facetList;
        int numElements;
      public:
        FacetListHelper();
        ~FacetListHelper();
        void add(const char* facet);
        openbusidl::rs::FacetList_var getFacetList();
    };

  /**
  * \brief Auxilia na construo de uma lista de propriedades.
  */
    class PropertyListHelper {
      private:
        openbusidl::rs::PropertyList_var propertyList;
        int numElements;
      public:
        PropertyListHelper();
        ~PropertyListHelper();
        void add(
          const char* key,
          const char* value);
        openbusidl::rs::PropertyList_var getPropertyList();
    };

  /**
  * \brief Representa o servio de registro. 
  */
    class RegistryService {
      private:
      /**
      * Ponteiro para o servio de registro. 
      */
        openbusidl::rs::IRegistryService* rgs;
      public:
      
      /**
      * Cria um objeto que representa o servio de registro.
      *
      * @param[in] _rgs Stub do servio de registro. 
      */
        RegistryService(openbusidl::rs::IRegistryService* _rgs);

      /**
      * Busca servios.
      * @param[in] facets Lista de facetas.
      * @return A lista de ofertas de servio que implementam as facetas 
      *   discriminadas no parmetro facets.
      */
        ServiceOfferList* find(FacetList facets);

      /**
      * Busca servios.
      * @param[in] facets Lista de facetas.
      * @param[in] criteria Lista de propriedades que descrevem o servio 
      *   desejado.
      * @return A lista de ofertas de servio que atendem aos critrios 
      *   especificados no parmetro criteria, e, implementam as facetas
      *   discriminadas no parmetro facets.
      */
        ServiceOfferList* findByCriteria(
          FacetList facets, 
          PropertyList criteria);

      /**
      * Registra um servio
      *
      * @param[in] serviceOffer Oferta do servio a ser registrado no 
      *    barramento.
      * @param[out] registryId Identificao do servio registrado.
      * @return Se o servio foi registrado com sucesso, o valor true  
      *   retornado, caso contrrio false  retornado.
      */
        bool Register(
          ServiceOffer serviceOffer,
          char*& registryId);

      /**
      * Remove uma oferta de servio.
      *
      * @param[in] registryId Identificador do registro da oferta de servio.
      * @return Se o servio foi removido com sucesso do barramento,
      *    o valor true  retornado, caso contrrio false  retornado.
      */
        bool unregister(char* registryId);
    };
  }
}

#endif
